MODULE IO;
(* =========================================================================  
   Slide Read functions using Astrobe 'Resource Data' 
  
   Target: Hardware-independent
   
   (c) 2009-2015 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

	IMPORT ResData, SYSTEM;

	VAR
		b1, b2: INTEGER;
    r: ResData.Resource;
    offset: INTEGER;
		c: INTEGER;
    data: INTEGER;

  PROCEDURE ReadByte*(): INTEGER;
  VAR
    byteNo: INTEGER;
  BEGIN
    byteNo := offset MOD 4;
    IF byteNo = 0 THEN 
      ResData.GetInt(r, offset DIV 4, data)
    END;
    INC(offset);
    (* Reverse byte ordering *)
    RETURN LSR(LSL(data, 8 * (3 - byteNo)), 24)
  END ReadByte;


  PROCEDURE ReadWord(): INTEGER;
  VAR
    b1, b2: INTEGER;
	BEGIN
		b1 := ReadByte();
    b2 := ReadByte();
		RETURN LSL(b2, 8) + b1
	END ReadWord;


	PROCEDURE ReadType* (): CHAR;
	BEGIN
		b1 := ReadByte();
		b2 := ReadByte();
		RETURN CHR(b2)
	END ReadType;


	PROCEDURE Offset(b: INTEGER): INTEGER;
  VAR
    offset: INTEGER;
  BEGIN
    IF b > 127 THEN offset := b - 256 ELSE offset := b END;
    RETURN offset
  END Offset;


  PROCEDURE GetOffset* (VAR o1, o2, o3, o4: INTEGER);
	BEGIN
		o1 := Offset(b1);
		o2 := Offset(ReadByte());
		o3 := Offset(ReadByte());
		o4 := Offset(ReadByte())
	END GetOffset;


	PROCEDURE GetColour* (): INTEGER;
	BEGIN
		RETURN b1
	END GetColour;


	PROCEDURE GetEndpoint* (VAR end1, end2: INTEGER);
	BEGIN
		end1 := Offset(b1);
		end2 := Offset(ReadByte())
	END GetEndpoint;


	PROCEDURE Get2Vector* (VAR v1, v2: INTEGER);
	BEGIN
		v1 := ReadWord();
		v2 := ReadWord();
		IF v2 < 0 THEN
			v2 := -v2;
		END
	END Get2Vector;


	PROCEDURE GetVector* (VAR v1, v2, v3, v4: INTEGER);
	BEGIN
		v1 := LSL(b2, 8) + b1;
		v2 := ReadWord();
		v3 := ReadWord();
		v4 := ReadWord()
	END GetVector;


	PROCEDURE ReadHeader* (VAR highX, highY: INTEGER);
  VAR
    i, b: INTEGER;
	BEGIN
    FOR i := 0 TO 18 DO b := ReadByte() END;
		highX := ReadWord();
		highY := ReadWord();
    FOR i := 23 TO 30 DO b := ReadByte() END
	END ReadHeader;


  PROCEDURE Reset*;
  BEGIN
    offset := 0
  END Reset;
  
  
  PROCEDURE Open*(name: ARRAY OF CHAR): BOOLEAN;
  VAR 
    ok: BOOLEAN;
	BEGIN
    ResData.Open(r, name);
    IF ResData.Size(r) > 0 THEN
      ok := TRUE;
      Reset()
    END;
    RETURN ok
	END Open;

END IO.
