MODULE Slide;
(* =========================================================================  
   Display an AutoCad Slide-format File 
  
   Target: Hardware-independent
   
   (c) 2009-2015 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

  IMPORT Trace := NoTrace, Graph, IO, Out;
 
  CONST
    offset    = 0FBX; (* -5 *)
    endfile   = 0FCX; (* -4 *)
    endPoint  = 0FEX; (* -2 *)
    newColour = 0FFX; (* -1 *)
    trace = TRUE;

  VAR
    c: CHAR;
    colour: INTEGER;
    scaleFactor: REAL;
    sf: REAL;
    hX, hY: INTEGER;
    panX, panY: INTEGER;
    Init*: PROCEDURE();

  PROCEDURE XScale (x: INTEGER): INTEGER;
  BEGIN
    RETURN FLOOR(FLT(x) * scaleFactor) - panX
  END XScale;

  PROCEDURE YScale (y: INTEGER): INTEGER;
  BEGIN
    RETURN Graph.MaxY - FLOOR(FLT(y) * scaleFactor) - panY  
  END YScale;

  PROCEDURE Draw(zoom: REAL; px, py: INTEGER);
  VAR
    drawLine: BOOLEAN;
    end1, end2, fromX, fromY, toX, toY: INTEGER;
    i, count: INTEGER;
    lastX, lastY: INTEGER;
    offset1, offset2, offset3, offset4: INTEGER;
  BEGIN
    IO.Reset();
    IO.ReadHeader(hX, hY);
    IF hX > hY THEN 
      scaleFactor := zoom * FLT(Graph.MaxX) / FLT(hX)
    ELSE 
      scaleFactor := zoom * FLT(Graph.MaxY) / FLT(hY) 
    END;
    panX := px;
    panY := py;
    c := IO.ReadType();
    lastX := 0;
    lastY := 0;
    i := 0;
    count := 0;
    REPEAT
      drawLine := TRUE;
      IF c = offset THEN 
          IO.GetOffset(offset1, offset2, offset3, offset4);
          Trace.Int4("GetOffset ", offset1, offset2, offset3, offset4);
          fromX := lastX + offset1;
          fromY := lastY + offset2;
          toX := lastX + offset3;
          toY := lastY + offset4;
          lastX := fromX;
          lastY := fromY;
      ELSIF c = endPoint THEN 
          fromX := lastX;
          fromY := lastY;
          IO.GetEndpoint(end1, end2);
          Trace.Int2("Endpoint  ", end1, end2);
          toX := lastX + end1;
          toY := lastY + end2;
          lastX := toX;
          lastY := toY;
      ELSIF c = newColour THEN 
          colour := IO.GetColour();
          Trace.Int1("NewColour ", colour);
          drawLine := FALSE
      ELSE
          IO.GetVector(fromX, fromY, toX, toY);
          Trace.Int4("GetVector ", fromX, fromY, toX, toY);
          lastX := fromX;
          lastY := fromY
      END;
      IF drawLine THEN
        Graph.Line(XScale(fromX), YScale(fromY), XScale(toX), YScale(toY), colour);
        Trace.Int4("Line ", fromX, fromY, toX, toY);
      END;
      c := IO.ReadType();
      INC(count)
    UNTIL (c = endfile)
  END Draw;

  PROCEDURE Display*(name: ARRAY OF CHAR; zoomd: REAL; panX, panY: INTEGER);
  BEGIN
    IF IO.Open(name) THEN
      Graph.ClearScreen;
      Draw(zoomd, panX, panY)
    ELSE
      Out.String(name); Out.String(" not found"); Out.Ln
    END 
  END Display;

BEGIN
  Init := Graph.Init;
END Slide.
